import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
*	Revisions: Convert from int to byte saved 10mb mem usage
*	Bench Mark: 
*		Board Type = byte
*		Time: 3.9s, 3.95s, (w/o gc)1.95s
*		Mem: 83,900K, 83,952K, (w/o gc) 81,056K
*	Revision: Convert from ArrayList to byte[]
*	Bench Mark:
*		Storage Type = byte[9]
*		Time: 1.75s,1.84s,1.87s
*		Mem: 66,000K, 64,596K,66,372K
*
*/
public class TTT extends JFrame
{
	JButton b[] = new JButton[9];
	JButton retry = new JButton("retry");
	JButton auto = new JButton("auto");
	byte[] board = new byte[9];
	boolean aiTurn = true;
	int move = -1;
	JLabel lwin = new JLabel("0");
	JLabel lloss = new JLabel("0");
	JLabel ltie = new JLabel("0");
	int win = 0;
	int loss = 0;
	int tie = 0;
	public TTT()
	{
		//setup board and UI
		setLayout(new BorderLayout());
		JPanel jpCenter = new JPanel(new GridLayout(3,3));
		JPanel jpSouth = new JPanel();
			jpSouth.add(retry);
			jpSouth.add(auto);
		JPanel jpNorth = new JPanel();
		jpNorth.add(lwin);
		jpNorth.add(new JLabel(":"));
		jpNorth.add(lloss);
		jpNorth.add(new JLabel(":"));
		jpNorth.add(ltie);
		for(int i = 0; i < 9; i++)
		{
			b[i] = new JButton();
			jpCenter.add(b[i]);
			board[i] = 0;
			b[i].addActionListener(new MoveListener(i));
		}
		retry.addActionListener(new MoveListener(-1));
		auto.addActionListener(new MoveListener(-2));
		getContentPane().add(jpCenter,BorderLayout.CENTER);
		getContentPane().add(jpSouth,BorderLayout.SOUTH);
		getContentPane().add(jpNorth,BorderLayout.NORTH);
		setSize(400,400);
		setVisible(true);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		//Tell computer to move
		moveAI(TTT.PLAYER_ONE);
		
	}
	public void moveAI(byte player)
	{
		System.out.println("Starting Timer");
		long t = System.currentTimeMillis();
		AI ai = new AI((byte[])board.clone(),(byte)(player * -1));
		System.out.println("Total Time: " + (System.currentTimeMillis() - t));
		byte temp[] = ai.getBoard();
		for(int i = 0; i < temp.length;i++)
		{
			if(temp[i] != board[i])
			{
				move = i;
				break;
			}
		}
		b[move].doClick();
		
	}
	
	public static byte checkWin(byte b[])
	{
		byte top[] = new byte[3];
		byte mid[] = new byte[3];
		byte bot[] = new byte[3];
		int numLeft = 0;
		for(int i = 0; i < 9;i++)
		{
			if(i < 3)
			top[i] = b[i];
			else if(i < 6)
			mid[(i)%3] = b[i];
			else
			bot[(i)%3] = b[i];
			if(b[i] == 0)
				numLeft++;
		}
		if(top[0] == top[1] && top[1] == top[2] && top[0] != 0)
			return top[0];
		if(mid[0] == mid[1] && mid[1] == mid[2] && mid[0] != 0)
			return mid[0];
		if(bot[0] == bot[1] && bot[1] == bot[2] && bot[0] != 0)
			return bot[0];
			
		if(top[0] == mid[1] && mid[1] == bot[2] && bot[2] != 0)
			return bot[2];
		if(top[2] == mid[1] && mid[1] == bot[0] && bot[0] != 0)
			return bot[0];
		
		if(top[0] == mid[0] && mid[0] == bot[0] && mid[0] != 0)
			return mid[0];
		if(top[1] == mid[1] && mid[1] == bot[1] && mid[1] != 0)
			return mid[1];
		if(top[2] == mid[2] && mid[2] == bot[2] && mid[2] != 0)
			return mid[2];
		if(numLeft == 0)
			return TTT.TIE;
		return 0;
	}
	public void displayBoard(int b[])
	{
		System.out.println(board[0] +""+ board[1] +""+ board[2] +"\n"+ board[3] +""+ board[4] +""+ board[5] +"\n"+ board[6] +""+ board[7] +""+ board[8]+"\n\n");
	}
	
	class MoveListener implements ActionListener
	{
		//sorry for the lac of constants but its just a quick example
		int place;
		public MoveListener(int place) { this.place = place;}
		public void actionPerformed(ActionEvent ae)
		{
			//if the button clicked was on the board
			if(place >= 0)
			{
				//if the AI clicked a button, then update the board
				if(aiTurn)
				{
					board[place] = TTT.PLAYER_ONE;
					b[place].setText("x");
					b[place].setEnabled(false);
				}
				else
				{
					//else your turn
					board[place] = TTT.PLAYER_TWO;
					b[place].setText("o");
					b[place].setEnabled(false);
					
				}
				aiTurn = !aiTurn;
				byte result = checkWin(board);
				
				switch(result)
				{
					//adding win to computer and update loss for you
					case TTT.PLAYER_ONE:
					System.out.println("adding win");
					loss++;
					lloss.setText("" + loss);
					for(int i = 0;i < 9;i++)
					{
						b[i].setEnabled(false);
					}
					break;
					case TTT.PLAYER_TWO:
					System.out.println("adding loss");
					win++;
					lwin.setText("" + win);
					for(int i = 0;i < 9;i++)
					{
						b[i].setEnabled(false);
					}
					break;
					case TTT.TIE:
					System.out.println("adding tie");
					tie++;
					ltie.setText("" + tie);
					for(int i = 0;i < 9;i++)
					{
						b[i].setEnabled(false);
					}
					break;
					default:
					if(aiTurn)
						moveAI(TTT.PLAYER_ONE);
//					else //uncomment if you want AI vs AI
//						moveAI(TTT.PLAYER_TWO);	
				}
			}
			else
			{
				//if reset was pressed
				if(place == -1)
				{
					aiTurn = true;
					for(int i = 0;i < 9;i++)
					{
						board[i] = 0;
						b[i].setText("");
						b[i].setEnabled(true);
					}
					System.out.println("reset");
					moveAI(TTT.PLAYER_ONE);
				}
				else
				{
					//else auto was pressed
					moveAI(TTT.PLAYER_TWO);
				}
			}
		}
	}
	public static final byte PLAYER_ONE = 1;
	public static final byte PLAYER_TWO = -1;
	public static final int MAX_DEPTH = 3;
	public static final byte TIE = 100;
	public static void main(String args[])
	{
		new TTT();
	}
}